@@ -66,7 +66,12 @@ AudioShortcutsService::AudioShortcutsService(QObject *parent, const QList<QVaria
             showVolume(volumePercent(sink->volume()));
         });
     });
+
+    // For global mute, we need to track sinks/sources and changes to them
     connect(m_sinkModel, &PulseAudioQt::SinkModel::rowsInserted, this, &AudioShortcutsService::handleNewSink);
+    connect(m_sourceModel, &PulseAudioQt::SinkModel::rowsInserted, this, &AudioShortcutsService::handleNewSource);
+    connect(m_sinkModel, &PulseAudioQt::SinkModel::dataChanged, this, &AudioShortcutsService::handleSinkDataChanged);
+    connect(m_sourceModel, &PulseAudioQt::SourceModel::dataChanged, this, &AudioShortcutsService::handleSourceDataChanged);
 
     QList<QAction *> actions;
 
@@ -164,10 +169,10 @@ AudioShortcutsService::AudioShortcutsService(QObject *parent, const QList<QVaria
     muteAction->setText(i18n("Mute"));
     muteAction->setShortcut(Qt::Key_VolumeMute);
     connect(muteAction, &QAction::triggered, this, [this]() {
-        if (m_globalConfig->globalMute()) {
-            disableGlobalMute();
+        if (m_globalConfig->globalMuteSinks()) {
+            disableGlobalSinkMute();
         } else {
-            enableGlobalMute();
+            enableGlobalSinkMute();
         }
     });
 
@@ -177,13 +182,11 @@ AudioShortcutsService::AudioShortcutsService(QObject *parent, const QList<QVaria
     muteMicAction->setText(i18n("Mute Microphone"));
     muteMicAction->setShortcuts({Qt::Key_MicMute, Qt::MetaModifier | Qt::Key_VolumeMute});
     connect(muteMicAction, &QAction::triggered, this, [this]() {
-        auto defaultSource = PulseAudioQt::Context::instance()->server()->defaultSource();
-        if (!defaultSource) {
-            return;
+        if (m_globalConfig->globalMuteSources()) {
+            disableGlobalSourceMute();
+        } else {
+            enableGlobalSourceMute();
         }
-        const bool toMute = !defaultSource->isMuted();
-        defaultSource->setMuted(toMute);
-        showMicMute(toMute ? 0 : volumePercent(defaultSource->volume()));
     });
 
     for (const auto action : actions) {
@@ -309,15 +312,6 @@ void AudioShortcutsService::handleDefaultSinkChange()
     m_osdDBusInterface->showText(icon, description);
 }
 
-void AudioShortcutsService::handleNewSink()
-{
-    if (m_globalConfig->globalMute()) {
-        for (int i = 0; i < m_sinkModel->rowCount(); i++) {
-            m_sinkModel->setData(m_sinkModel->index(i, 0), true, m_sinkModel->role("Muted"_ba));
-        }
-    }
-}
-
 void AudioShortcutsService::muteVolume()
 {
     auto sink = m_preferredDevice.sink();
@@ -325,11 +319,11 @@ void AudioShortcutsService::muteVolume()
         return;
     }
     if (!sink->isMuted()) {
-        enableGlobalMute();
+        enableGlobalSinkMute();
         showMute(0);
     } else {
-        if (m_globalConfig->globalMute()) {
-            disableGlobalMute();
+        if (m_globalConfig->globalMuteSinks()) {
+            disableGlobalSinkMute();
         }
         sink->setMuted(false);
         showMute(sink->volume());
@@ -337,51 +331,6 @@ void AudioShortcutsService::muteVolume()
     }
 }
 
-void AudioShortcutsService::enableGlobalMute()
-{
-    QStringList alreadyMutedDevices;
-    for (int i = 0; i < m_sinkModel->rowCount(); i++) {
-        const auto idx = m_sinkModel->index(i, 0);
-        const bool isAlreadyMuted = m_sinkModel->data(idx, m_sinkModel->role("Muted"_ba)).toBool();
-        const QString name = m_sinkModel->data(idx, m_sinkModel->role("Name"_ba)).toString();
-        const QString activePortIdx = QString::number(m_sinkModel->data(idx, m_sinkModel->role("ActivePortIndex"_ba)).toUInt());
-        if (isAlreadyMuted) {
-            alreadyMutedDevices.append(name % "." % activePortIdx);
-        } else {
-            m_sinkModel->setData(idx, true, m_sinkModel->role("Muted"_ba));
-        }
-    }
-    if (alreadyMutedDevices.length() == m_sinkModel->rowCount()) {
-        alreadyMutedDevices.clear();
-    }
-    m_globalConfig->setGlobalMuteDevices(alreadyMutedDevices);
-    m_globalConfig->setGlobalMute(true);
-    m_globalConfig->save();
-    showMute(0);
-}
-
-void AudioShortcutsService::disableGlobalMute()
-{
-    QStringList keepMutedDevices = m_globalConfig->globalMuteDevices();
-    for (int i = 0; i < m_sinkModel->rowCount(); i++) {
-        const auto idx = m_sinkModel->index(i, 0);
-        const QString name = m_sinkModel->data(idx, m_sinkModel->role("Name"_ba)).toString();
-        const QString activePortIdx = QString::number(m_sinkModel->data(idx, m_sinkModel->role("ActivePortIndex"_ba)).toUInt());
-        const QString deviceId = name % "." % activePortIdx;
-        if (!keepMutedDevices.contains(deviceId)) {
-            m_sinkModel->setData(idx, false, m_sinkModel->role("Muted"_ba));
-        }
-    }
-    m_globalConfig->setGlobalMute(false);
-    m_globalConfig->setGlobalMuteDevices({});
-    m_globalConfig->save();
-    auto sink = m_preferredDevice.sink();
-    if (sink) {
-        showMute(volumePercent(sink->volume()));
-        playFeedback(-1);
-    }
-}
-
 void AudioShortcutsService::playFeedback(int sinkIdx)
 {
     if (!m_globalConfig->audioFeedback()) {
@@ -426,6 +375,196 @@ void AudioShortcutsService::showMicMute(int percent)
     m_osdDBusInterface->microphoneVolumeChanged(percent);
 }
 
+void AudioShortcutsService::applyGlobalSinkMute(const QModelIndex &index, const bool globalMute, QStringList &globalMuteMutedDevices)
+{
+    const QString name = m_sinkModel->data(index, m_sinkModel->role("Name"_ba)).toString();
+    const QString activePortIdx = QString::number(m_sinkModel->data(index, m_sinkModel->role("ActivePortIndex"_ba)).toUInt());
+    const QString deviceId = name % "." % activePortIdx;
+
+    if (globalMute) {
+        // Mute the device if unmuted
+        const bool isUnmuted = !m_sinkModel->data(index, m_sinkModel->role("Muted"_ba)).toBool();
+        if (isUnmuted) {
+            if (!globalMuteMutedDevices.contains(deviceId)) {
+                globalMuteMutedDevices.append(deviceId);
+            }
+            m_sinkModel->setData(index, true, m_sinkModel->role("Muted"_ba));
+        }
+    } else {
+        // Unmute the device if global mute has muted it
+        if (globalMuteMutedDevices.removeAll(deviceId)) {
+            m_sinkModel->setData(index, false, m_sinkModel->role("Muted"_ba));
+        }
+    }
+}
+
+void AudioShortcutsService::applyGlobalSourceMute(const QModelIndex &index, const bool globalMute, QStringList &globalMicMuteMutedDevices)
+{
+    const QString name = m_sourceModel->data(index, m_sourceModel->role("Name"_ba)).toString();
+    const QString activePortIdx = QString::number(m_sourceModel->data(index, m_sourceModel->role("ActivePortIndex"_ba)).toUInt());
+    const QString deviceId = name % "." % activePortIdx;
+
+    if (globalMute) {
+        // Mute the device if unmuted
+        const bool isUnmuted = !m_sourceModel->data(index, m_sourceModel->role("Muted"_ba)).toBool();
+        if (isUnmuted) {
+            if (!globalMicMuteMutedDevices.contains(deviceId)) {
+                globalMicMuteMutedDevices.append(deviceId);
+            }
+            m_sourceModel->setData(index, true, m_sourceModel->role("Muted"_ba));
+        }
+    } else {
+        // Unmute the device if global mute has muted it
+        if (globalMicMuteMutedDevices.removeAll(deviceId)) {
+            m_sourceModel->setData(index, false, m_sourceModel->role("Muted"_ba));
+        }
+    }
+}
+
+void AudioShortcutsService::handleNewSink(const QModelIndex &parent, int first, int last)
+{
+    Q_UNUSED(parent)
+
+    QStringList globalMuteMutedDevices = m_globalConfig->globalMuteSinksMutedDevices();
+    const bool globalMute = m_globalConfig->globalMuteSinks();
+
+    for (int i = first; i <= last; ++i) {
+        applyGlobalSinkMute(m_sinkModel->index(i, 0), globalMute, globalMuteMutedDevices);
+    }
+
+    m_globalConfig->setGlobalMuteSinksMutedDevices(globalMuteMutedDevices);
+    m_globalConfig->save();
+}
+
+void AudioShortcutsService::handleNewSource(const QModelIndex &parent, int first, int last)
+{
+    Q_UNUSED(parent)
+
+    QStringList globalMicMuteMutedDevices = m_globalConfig->globalMuteSourcesMutedDevices();
+    const bool globalMicMute = m_globalConfig->globalMuteSources();
+
+    for (int i = first; i <= last; ++i) {
+        applyGlobalSourceMute(m_sinkModel->index(i, 0), globalMicMute, globalMicMuteMutedDevices);
+    }
+
+    m_globalConfig->setGlobalMuteSourcesMutedDevices(globalMicMuteMutedDevices);
+    m_globalConfig->save();
+}
+
+void AudioShortcutsService::handleSinkDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles)
+{
+    QStringList globalMuteMutedDevices = m_globalConfig->globalMuteSinksMutedDevices();
+    const bool globalMute = m_globalConfig->globalMuteSinks();
+
+    for (int i = topLeft.row(); i <= bottomRight.row(); ++i) {
+        const auto index = m_sinkModel->index(i, 0);
+
+        const bool activePortIndexChanged = roles.contains(m_sinkModel->role("ActivePortIndex"_ba));
+        const bool volumeChanged = roles.contains(m_sinkModel->role("Volume"_ba));
+        const bool mutedChanged = roles.contains(m_sinkModel->role("Muted"_ba));
+        const bool isUnmuted = !m_sinkModel->data(index, m_sinkModel->role("Muted"_ba)).toBool();
+
+        if (activePortIndexChanged) {
+            // Apply global mute to new device
+            applyGlobalSinkMute(index, globalMute, globalMuteMutedDevices);
+
+            m_globalConfig->setGlobalMuteSinksMutedDevices(globalMuteMutedDevices);
+            m_globalConfig->save();
+        } else if (globalMute && (volumeChanged || (mutedChanged && isUnmuted))) {
+            // Changes to volume or unmuting break global mute
+            disableGlobalSinkMute();
+        }
+    }
+}
+
+void AudioShortcutsService::handleSourceDataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight, const QList<int> &roles)
+{
+    QStringList globalMicMuteMutedDevices = m_globalConfig->globalMuteSourcesMutedDevices();
+    const bool globalMicMute = m_globalConfig->globalMuteSources();
+
+    for (int i = topLeft.row(); i <= bottomRight.row(); ++i) {
+        const auto index = m_sourceModel->index(i, 0);
+
+        const bool activePortIndexChanged = roles.contains(m_sourceModel->role("ActivePortIndex"_ba));
+        const bool volumeChanged = roles.contains(m_sourceModel->role("Volume"_ba));
+        const bool mutedChanged = roles.contains(m_sourceModel->role("Muted"_ba));
+        const bool isUnmuted = !m_sourceModel->data(index, m_sourceModel->role("Muted"_ba)).toBool();
+
+        if (activePortIndexChanged) {
+            // Apply global mute to new device
+            applyGlobalSourceMute(index, globalMicMute, globalMicMuteMutedDevices);
+
+            m_globalConfig->setGlobalMuteSourcesMutedDevices(globalMicMuteMutedDevices);
+            m_globalConfig->save();
+        } else if (globalMicMute && (volumeChanged || (mutedChanged && isUnmuted))) {
+            // Changes to volume or unmuting break global mute
+            disableGlobalSourceMute();
+        }
+    }
+}
+
+void AudioShortcutsService::enableGlobalSinkMute()
+{
+    QStringList globalMuteMutedDevices = m_globalConfig->globalMuteSinksMutedDevices();
+    for (int i = 0; i < m_sinkModel->rowCount(); ++i) {
+        applyGlobalSinkMute(m_sinkModel->index(i, 0), true, globalMuteMutedDevices);
+    }
+
+    m_globalConfig->setGlobalMuteSinksMutedDevices(globalMuteMutedDevices);
+    m_globalConfig->setGlobalMuteSinks(true);
+    m_globalConfig->save();
+
+    showMute(0);
+}
+
+void AudioShortcutsService::enableGlobalSourceMute()
+{
+    QStringList globalMicMuteMutedDevices = m_globalConfig->globalMuteSourcesMutedDevices();
+    for (int i = 0; i < m_sourceModel->rowCount(); ++i) {
+        applyGlobalSourceMute(m_sourceModel->index(i, 0), true, globalMicMuteMutedDevices);
+    }
+
+    m_globalConfig->setGlobalMuteSourcesMutedDevices(globalMicMuteMutedDevices);
+    m_globalConfig->setGlobalMuteSources(true);
+    m_globalConfig->save();
+
+    showMicMute(0);
+}
+
+void AudioShortcutsService::disableGlobalSinkMute()
+{
+    QStringList globalMuteMutedDevices = m_globalConfig->globalMuteSinksMutedDevices();
+    for (int i = 0; i < m_sinkModel->rowCount(); ++i) {
+        applyGlobalSinkMute(m_sinkModel->index(i, 0), false, globalMuteMutedDevices);
+    }
+
+    m_globalConfig->setGlobalMuteSinksMutedDevices(globalMuteMutedDevices);
+    m_globalConfig->setGlobalMuteSinks(false);
+    m_globalConfig->save();
+
+    const auto sink = m_preferredDevice.sink();
+    if (sink) {
+        showMute(volumePercent(sink->volume()));
+        playFeedback(-1);
+    }
+}
+
+void AudioShortcutsService::disableGlobalSourceMute()
+{
+    QStringList globalMicMuteMutedDevices = m_globalConfig->globalMuteSourcesMutedDevices();
+    for (int i = 0; i < m_sourceModel->rowCount(); ++i) {
+        applyGlobalSourceMute(m_sourceModel->index(i, 0), false, globalMicMuteMutedDevices);
+    }
+
+    m_globalConfig->setGlobalMuteSourcesMutedDevices(globalMicMuteMutedDevices);
+    m_globalConfig->setGlobalMuteSources(false);
+    m_globalConfig->save();
+
+    const auto source = m_preferredDevice.source();
+    if (source)
+        showMicMute(volumePercent(source->volume()));
+}
+
 #include "audioshortcutsservice.moc"
 
 #include "moc_audioshortcutsservice.cpp"
