@@ -15,11 +15,16 @@ using namespace Qt::Literals::StringLiterals;
 constexpr QLatin1String OSD_DBUS_SERVICE = "org.kde.plasmashell"_L1;
 constexpr QLatin1String OSD_DBUS_PATH = "/org/kde/osdService"_L1;
 
-void GlobalService::globalMute()
+void GlobalService::globalMuteSinks()
 {
     invokeShortcut(u"mute"_s);
 }
 
+void GlobalService::globalMuteSources()
+{
+    invokeShortcut(u"mic_mute"_s);
+}
+
 void GlobalService::volumeUp()
 {
     invokeShortcut(u"increase_volume"_s);
